
import Numeric

import geompy
import SALOMEDS
import salome
import smesh

import StdMeshers
import NETGENPlugin

from SMESH import *
from StdMeshers import *


## import GEOM dump file ## 
import string, os, sys, re


zeta = 0.5   # changed from 0.3 - to make it symmetric
zetam1 = 1-zeta
Bx = 3
By = 8
Bz = 23
block = geompy.MakeBoxDXDYDZ(Bx,By,Bz)


block26 = geompy.GetSubShape(block, [26])
p1star = geompy.MakeVertexOnCurve(block26,zeta)
block30 = geompy.GetSubShape(block, [30])
p2star = geompy.MakeVertexOnCurve(block30,zetam1)

# to be used in the mesher
x1,y1,z1 = geompy.PointCoordinates(p1star)
x2,y2,z2 = geompy.PointCoordinates(p2star)

DZplus = 17.0
DZmin  =  1.2 # create overlap of shell into volume block for Arlequin method
DZmin  =  0.0 # no overlap needed for LIAISON_MAIL, COQUE_MASSIF or MASSIF_COQUE


p1 = geompy.MakeVertexWithRef(p1star, 0, 0, -DZmin)
p2 = geompy.MakeVertexWithRef(p2star, 0, 0, -DZmin)
p3 = geompy.MakeVertexWithRef(p2star, 0, 0, DZplus)
p4 = geompy.MakeVertexWithRef(p1star, 0, 0, DZplus)

line1 = geompy.MakeLineTwoPnt(p1, p4)
line2 = geompy.MakeLineTwoPnt(p4, p3)
line3 = geompy.MakeLineTwoPnt(p3, p2)
line4 = geompy.MakeLineTwoPnt(p2, p1)
face1 = geompy.MakeFaceWires([line1, line2, line3, line4], 1)


# create groups (Faces) at top  and bottom of block
FBbot = geompy.CreateGroup(block, geompy.ShapeType["FACE"])
geompy.UnionIDs(FBbot, [31])
block = geompy.GetMainShape(FBbot)
FBtop = geompy.CreateGroup(block, geompy.ShapeType["FACE"])
geompy.UnionIDs(FBtop, [33])
block = geompy.GetMainShape(FBtop)


# create groups (edges) at top and bottom of face1
EFbot = geompy.CreateGroup(face1, geompy.ShapeType["EDGE"])
geompy.UnionIDs(EFbot, [6])
face1 = geompy.GetMainShape(EFbot)
EFtop = geompy.CreateGroup(face1, geompy.ShapeType["EDGE"])
geompy.UnionIDs(EFtop, [10])
face1 = geompy.GetMainShape(EFtop)

# create groups (shell and node) on faces face1-2
shell = geompy.CreateGroup(face1, geompy.ShapeType["FACE"])
geompy.UnionIDs(shell, [1])
face1 = geompy.GetMainShape(shell)

# make entities visible
geompy.addToStudy( p1, "p1" )
geompy.addToStudy( p2, "p2" )
geompy.addToStudy( p3, "p3" )
geompy.addToStudy( p4, "p4" )

geompy.addToStudy( line1, "line1" )
geompy.addToStudy( line2, "line2" )
geompy.addToStudy( line3, "line3" )
geompy.addToStudy( line4, "line4" )


# create compounds of quadconstr <-- face1 and blcok
quadconstr = geompy.MakeCompound([face1, block])

# define group on quadconstr
Cline = geompy.CreateGroup(quadconstr, geompy.ShapeType["EDGE"])
geompy.UnionIDs(Cline, [7])
quadconstr = geompy.GetMainShape(Cline)
Ctop = geompy.CreateGroup(quadconstr, geompy.ShapeType["FACE"])
geompy.UnionIDs(Ctop, [44])
quadconstr = geompy.GetMainShape(Ctop)
Cbot = geompy.CreateGroup(quadconstr, geompy.ShapeType["FACE"])
geompy.UnionIDs(Cbot, [42])
quadconstr = geompy.GetMainShape(Cbot)

Nforce = geompy.CreateGroup(quadconstr, geompy.ShapeType["VERTEX"])
geompy.UnionIDs(Nforce, [5, 10])
quadconstr = geompy.GetMainShape(Nforce)


geompy.addToStudy( quadconstr, "quadconstr" )
geompy.addToStudy( face1, "face1" )
geompy.addToStudy( block, "block" )

geompy.addToStudyInFather(quadconstr,block, "block" )
geompy.addToStudyInFather(quadconstr,shell, "shell" )
geompy.addToStudyInFather(quadconstr,FBtop, "FBtop" )
geompy.addToStudyInFather(quadconstr,FBbot, "FBbot" )
geompy.addToStudyInFather(quadconstr,Ctop, "Ctop" )
geompy.addToStudyInFather(quadconstr,Cbot, "Cbot" )
geompy.addToStudyInFather(quadconstr,Cline, "Cline" )
geompy.addToStudyInFather(quadconstr,Nforce, "Nforce" )




### ============
### start mesher
### ============

quadconstr = smesh.Mesh(quadconstr)
NETGEN_3D_Parameters = smesh.CreateHypothesis('NETGEN_Parameters', 'NETGENEngine')
NETGEN_3D_Parameters.SetMaxSize( 0.4 )
NETGEN_3D_Parameters.SetSecondOrder( 1 )
NETGEN_3D_Parameters.SetOptimize( 1 )
NETGEN_3D_Parameters.SetFineness( 3 )
Max_Size3D = smesh.CreateHypothesis('MaxLength')
Max_Size3D.SetLength( 1.0 )

NETGEN_2D_Parameters = smesh.CreateHypothesis('NETGEN_Parameters_2D', 'NETGENEngine')
NETGEN_2D_Parameters.SetQuadAllowed( 0 )
Max_Size2D = smesh.CreateHypothesis('MaxLength')
Max_Size2D.SetLength( 1.0 )
Regular1D = quadconstr.Segment()
Automatic_length = Regular1D.AutomaticLength(0.6)
MEFISTO_2D = quadconstr.Triangle()
Tetrahedron_Netgen = quadconstr.Tetrahedron(algo=smesh.NETGEN)
isDone = quadconstr.Compute()


###=====================================
### add groups to mesh for loads, bc, materialprops
block = quadconstr.Group(block)
shell = quadconstr.Group(shell)
FBbot = quadconstr.Group(FBbot)
FBtop = quadconstr.Group(FBtop)
Cline = quadconstr.Group(Cline)
Nforce = quadconstr.Group(Nforce)



## set object names
smesh.SetName(quadconstr.GetMesh(), 'quadconstr')
smesh.SetName(NETGEN_3D_Parameters, 'NETGEN 3D Parameters')
smesh.SetName(Max_Size3D, 'Max Size 3D')
smesh.SetName(NETGEN_2D_Parameters, 'NETGEN 2D Parameters')
smesh.SetName(Max_Size2D, 'Max Size 2D')
smesh.SetName(Regular1D.GetAlgorithm(), 'Regular 1D')
smesh.SetName(Automatic_length, 'Automatic length')
#smesh.SetName(Quadratic_Mesh, 'Quadratic Mesh')
#smesh.SetName(MEFISTO_2D.GetAlgorithm(), 'MEFISTO_2D')
smesh.SetName(Tetrahedron_Netgen.GetAlgorithm(), 'Tetrahedron (Netgen)')

quadconstr.ConvertToQuadratic( 0 ) ## medium nodes on geometry 0: selected; 1: not selected
#quadconstr.ConvertFromQuadratic()

smesh.SetName(block, 'block')
smesh.SetName(shell, 'shell')
smesh.SetName(FBbot, 'FBbot')
smesh.SetName(FBtop, 'FBtop')
smesh.SetName(Nforce, 'Nforce')


def PrintMeshInfo(quadconstr):
     #aMesh = theMesh.GetMesh()
     print "mesh information:"
     print "Number of nodes       : ", quadconstr.NbNodes()
     print "Number of edges       : ", quadconstr.NbEdges()
     print "Number of faces       : ", quadconstr.NbFaces()
     print "Number of volumes     : ", quadconstr.NbVolumes()



number_of_volumes = quadconstr.NbVolumes()
list_of_volumes = quadconstr.GetElementsByType(VOLUME)   #ALL, NODE, EDGE, FACE, VOLUME (enumaration)
list_of_faces = quadconstr.GetElementsByType(FACE)   #ALL, NODE, EDGE, FACE, VOLUME (enumaration)
PrintMeshInfo(quadconstr)


if salome.sg.hasDesktop():
	salome.sg.updateObjBrowser(0)




